package dev.fyloz.colorrecipesexplorer.repository

import dev.fyloz.colorrecipesexplorer.model.account.Group
import dev.fyloz.colorrecipesexplorer.model.account.GroupToken
import dev.fyloz.colorrecipesexplorer.model.account.User
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.data.jpa.repository.Query
import org.springframework.stereotype.Repository
import java.util.*

/**
 * Default group users are deprecated and should not be used anymore.
 * To prevent data loss, they will not be removed from the database,
 * but they are excluded from results from the database.
 */
@Repository
interface UserRepository : JpaRepository<User, Long> {
    fun findAllByIsDefaultGroupUserIsFalse(): MutableList<User>

    fun findByIdAndIsDefaultGroupUserIsFalse(id: Long): User?

    /** Checks if a user with the given [firstName], [lastName] and a different [id] exists. */
    fun existsByFirstNameAndLastNameAndIdNotAndIsDefaultGroupUserIsFalse(firstName: String, lastName: String, id: Long): Boolean

    /** Finds all users for the given [group]. */
    @Query("SELECT u FROM User u WHERE u.group = :group AND u.isSystemUser IS FALSE AND u.isDefaultGroupUser IS FALSE")
    fun findAllByGroup(group: Group): Collection<User>

    /** Finds the user with the given [firstName] and [lastName]. */
    @Query("SELECT u From User u WHERE u.firstName = :firstName AND u.lastName = :lastName")
    fun findByFirstNameAndLastName(firstName: String, lastName: String): User?
}

@Repository
interface GroupRepository : JpaRepository<Group, Long> {
    /** Checks if a group with the given [name] and a different [id] exists. */
    fun existsByNameAndIdNot(name: String, id: Long): Boolean
}

@Repository
interface GroupTokenRepository : JpaRepository<GroupToken, UUID> {
    /** Checks if a token with the given [name] exists. */
    fun existsByName(name: String): Boolean
}
