package dev.fyloz.colorrecipesexplorer.rest.account

import dev.fyloz.colorrecipesexplorer.Constants
import dev.fyloz.colorrecipesexplorer.config.annotations.PreAuthorizeEditUsers
import dev.fyloz.colorrecipesexplorer.config.annotations.PreAuthorizeViewUsers
import dev.fyloz.colorrecipesexplorer.dtos.account.GroupDto
import dev.fyloz.colorrecipesexplorer.logic.account.GroupLogic
import dev.fyloz.colorrecipesexplorer.logic.account.JwtLogic
import dev.fyloz.colorrecipesexplorer.logic.account.UserLogic
import dev.fyloz.colorrecipesexplorer.rest.created
import dev.fyloz.colorrecipesexplorer.rest.noContent
import dev.fyloz.colorrecipesexplorer.rest.ok
import org.springframework.context.annotation.Profile
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.web.bind.annotation.*
import javax.servlet.http.HttpServletRequest
import javax.servlet.http.HttpServletResponse
import javax.validation.Valid


@RestController
@RequestMapping(Constants.ControllerPaths.GROUP)
@Profile("!emergency")
class GroupController(
    private val groupLogic: GroupLogic
) {
    @GetMapping
    @PreAuthorize("hasAnyAuthority('VIEW_RECIPES', 'VIEW_USERS')")
    fun getAll() =
        ok(groupLogic.getAll())

    @GetMapping("{id}")
    @PreAuthorizeViewUsers
    fun getById(@PathVariable id: Long) =
        ok(groupLogic.getById(id))

    @GetMapping("{id}/users")
    @PreAuthorizeViewUsers
    fun getUsersForGroup(@PathVariable id: Long) =
        ok(groupLogic.getUsersForGroup(id))

    @PostMapping
    @PreAuthorizeEditUsers
    fun save(@Valid @RequestBody group: GroupDto) =
        created<GroupDto>(Constants.ControllerPaths.GROUP) {
            groupLogic.save(group)
        }

    @PutMapping
    @PreAuthorizeEditUsers
    fun update(@Valid @RequestBody group: GroupDto) =
        noContent {
            groupLogic.update(group)
        }

    @DeleteMapping("{id}")
    @PreAuthorizeEditUsers
    fun deleteById(@PathVariable id: Long) =
        noContent {
            groupLogic.deleteById(id)
        }
}
