package dev.fyloz.colorrecipesexplorer.logic.account

import dev.fyloz.colorrecipesexplorer.dtos.account.GroupDto
import dev.fyloz.colorrecipesexplorer.dtos.account.UserDto
import dev.fyloz.colorrecipesexplorer.exception.AlreadyExistsException
import dev.fyloz.colorrecipesexplorer.service.account.GroupService
import io.mockk.*
import org.junit.jupiter.api.AfterEach
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.assertThrows

class DefaultGroupLogicTest {
    private val group = GroupDto(1L, "Unit test group", listOf())
    private val user = UserDto(1L, "Unit test", "User", "asecurepassword", null, listOf())

    private val groupServiceMock = mockk<GroupService> {
        every { existsById(any()) } returns false
        every { existsByName(any(), any()) } returns false
        every { getAll() } returns listOf()
        every { getById(any()) } returns group
        every { save(any()) } returns group
        every { deleteById(any()) } just runs
    }
    private val userLogicMock = mockk<UserLogic> {
        every { getAllByGroup(any()) } returns listOf()
        every { getById(any(), any()) } returns user
        every { deleteById(any()) } just runs
    }

    private val groupLogic = spyk(DefaultGroupLogic(groupServiceMock, userLogicMock))

    @AfterEach
    internal fun afterEach() {
        clearAllMocks()
    }

    @Test
    fun getUsersForGroup_normalBehavior_callsGetAllByGroupInUserLogic() {
        // Arrange
        every { groupLogic.getById(any()) } returns group

        // Act
        groupLogic.getUsersForGroup(group.id)

        // Assert
        verify {
            userLogicMock.getAllByGroup(group)
        }
        confirmVerified(userLogicMock)
    }

    @Test
    fun save_nameAlreadyExists_throwsAlreadyExists() {
        // Arrange
        every { groupServiceMock.existsByName(any(), any()) } returns true

        // Act
        // Assert
        assertThrows<AlreadyExistsException> { groupLogic.save(group) }
    }

    @Test
    fun update_normalBehavior_throwsAlreadyExists() {
        // Arrange
        every { groupServiceMock.existsByName(any(), any()) } returns true

        // Act
        // Assert
        assertThrows<AlreadyExistsException> { groupLogic.update(group) }
    }
}
